@@ -27,6 +27,13 @@
 
 #include <string.h>
 
+/**
+ * GWeatherLocationEntry:
+ *
+ * A subclass of #GtkEntry that provides autocompletion on
+ * #GWeatherLocation<!-- -->s
+ **/
+
 G_DEFINE_TYPE (GWeatherLocationEntry, gweather_location_entry, GTK_TYPE_ENTRY)
 
 enum {
@@ -60,6 +67,7 @@ static gboolean match_selected (GtkEntryCompletion *completion,
 				GtkTreeModel       *model,
 				GtkTreeIter        *iter,
 				gpointer            entry);
+static void     entry_changed (GWeatherLocationEntry *entry);
 
 static void
 gweather_location_entry_init (GWeatherLocationEntry *entry)
@@ -77,6 +85,10 @@ gweather_location_entry_init (GWeatherLocationEntry *entry)
 
     gtk_entry_set_completion (GTK_ENTRY (entry), completion);
     g_object_unref (completion);
+
+    entry->custom_text = FALSE;
+    g_signal_connect (entry, "changed",
+		      G_CALLBACK (entry_changed), NULL);
 }
 
 static void
@@ -151,6 +163,12 @@ get_property (GObject *object, guint prop_id,
     }
 }
 
+static void
+entry_changed (GWeatherLocationEntry *entry)
+{
+    entry->custom_text = TRUE;
+}
+
 static void
 set_location_internal (GWeatherLocationEntry *entry,
 		       GtkTreeModel          *model,
@@ -169,16 +187,27 @@ set_location_internal (GWeatherLocationEntry *entry,
 			    -1);
 	entry->location = gweather_location_ref (loc);
 	gtk_entry_set_text (GTK_ENTRY (entry), name);
+	entry->custom_text = FALSE;
 	g_free (name);
     } else {
 	entry->location = NULL;
 	gtk_entry_set_text (GTK_ENTRY (entry), "");
+	entry->custom_text = TRUE;
     }
 
     gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
     g_object_notify (G_OBJECT (entry), "location");
 }
 
+/**
+ * gweather_location_entry_set_location:
+ * @entry: a #GWeatherLocationEntry
+ * @loc: (allow-none): a #GWeatherLocation in @entry, or %NULL to
+ * clear @entry
+ *
+ * Sets @entry's location to @loc, and updates the text of the
+ * entry accordingly.
+ **/
 void
 gweather_location_entry_set_location (GWeatherLocationEntry *entry,
 				      GWeatherLocation      *loc)
@@ -207,6 +236,17 @@ gweather_location_entry_set_location (GWeatherLocationEntry *entry,
     set_location_internal (entry, model, NULL);
 }
 
+/**
+ * gweather_location_entry_get_location:
+ * @entry: a #GWeatherLocationEntry
+ *
+ * Gets the location that was set by a previous call to
+ * gweather_location_entry_set_location() or was selected by the user.
+ *
+ * Return value: (transfer full) (allow-none): the selected location
+ * (which you must unref when you are done with it), or %NULL if no
+ * location is selected.
+ **/
 GWeatherLocation *
 gweather_location_entry_get_location (GWeatherLocationEntry *entry)
 {
@@ -218,7 +258,39 @@ gweather_location_entry_get_location (GWeatherLocationEntry *entry)
 	return NULL;
 }
 
-void
+/**
+ * gweather_location_entry_has_custom_text:
+ * @entry: a #GWeatherLocationEntry
+ *
+ * Checks whether or not @entry's text has been modified by the user.
+ * Note that this does not mean that no location is associated with @entry.
+ * gweather_location_entry_get_location() should be used for this.
+ *
+ * Return value: %TRUE if @entry's text was modified by the user, or %FALSE if
+ * it's set to the default text of a location.
+ **/
+gboolean
+gweather_location_entry_has_custom_text (GWeatherLocationEntry *entry)
+{
+    g_return_val_if_fail (GWEATHER_IS_LOCATION_ENTRY (entry), FALSE);
+
+    return entry->custom_text;
+}
+
+/**
+ * gweather_location_entry_set_city:
+ * @entry: a #GWeatherLocationEntry
+ * @city_name: (allow-none): the city name, or %NULL
+ * @code: the METAR station code
+ *
+ * Sets @entry's location to a city with the given @code, and given
+ * @city_name, if non-%NULL. If there is no matching city, sets
+ * @entry's location to %NULL.
+ *
+ * Return value: %TRUE if @entry's location could be set to a matching city,
+ * %FALSE otherwise.
+ **/
+gboolean
 gweather_location_entry_set_city (GWeatherLocationEntry *entry,
 				  const char            *city_name,
 				  const char            *code)
@@ -230,8 +302,8 @@ gweather_location_entry_set_city (GWeatherLocationEntry *entry,
     const char *cmpcode;
     char *cmpname;
 
-    g_return_if_fail (GWEATHER_IS_LOCATION_ENTRY (entry));
-    g_return_if_fail (code != NULL);
+    g_return_val_if_fail (GWEATHER_IS_LOCATION_ENTRY (entry), FALSE);
+    g_return_val_if_fail (code != NULL, FALSE);
 
     completion = gtk_entry_get_completion (GTK_ENTRY (entry));
     model = gtk_entry_completion_get_model (completion);
@@ -256,10 +328,12 @@ gweather_location_entry_set_city (GWeatherLocationEntry *entry,
 	}
 
 	set_location_internal (entry, model, &iter);
-	return;
+	return TRUE;
     } while (gtk_tree_model_iter_next (model, &iter));
 
     set_location_internal (entry, model, NULL);
+
+    return FALSE;
 }
 
 static void
@@ -312,58 +386,10 @@ fill_location_entry_model (GtkTreeStore *store, GWeatherLocation *loc,
 	break;
 
     case GWEATHER_LOCATION_CITY:
-	if (children[0] && children[1]) {
-	    /* If there are multiple (<location>) children, add a line
-	     * for each of them.
-	     */
-	    for (i = 0; children[i]; i++) {
-		display_name = g_strdup_printf ("%s (%s), %s",
-						gweather_location_get_name (loc),
-						gweather_location_get_name (children[i]),
-						parent_display_name);
-		compare_name = g_strdup_printf ("%s (%s), %s",
-						gweather_location_get_sort_name (loc),
-						gweather_location_get_sort_name (children[i]),
-						parent_compare_name);
+	if (!children[0])
+	    break;
 
-		gtk_tree_store_append (store, &iter, NULL);
-		gtk_tree_store_set (store, &iter,
-				    GWEATHER_LOCATION_ENTRY_COL_LOCATION, children[i],
-				    GWEATHER_LOCATION_ENTRY_COL_DISPLAY_NAME, display_name,
-				    GWEATHER_LOCATION_ENTRY_COL_COMPARE_NAME, compare_name,
-				    -1);
-
-		g_free (display_name);
-		g_free (compare_name);
-	    }
-	} else if (children[0]) {
-	    /* Else there's only one location. This is a mix of the
-	     * city-with-multiple-location case above and the
-	     * location-with-no-city case below.
-	     */
-	    display_name = g_strdup_printf ("%s, %s",
-					    gweather_location_get_name (loc),
-					    parent_display_name);
-	    compare_name = g_strdup_printf ("%s, %s",
-					    gweather_location_get_sort_name (loc),
-					    parent_compare_name);
-
-	    gtk_tree_store_append (store, &iter, NULL);
-	    gtk_tree_store_set (store, &iter,
-				GWEATHER_LOCATION_ENTRY_COL_LOCATION, children[0],
-				GWEATHER_LOCATION_ENTRY_COL_DISPLAY_NAME, display_name,
-				GWEATHER_LOCATION_ENTRY_COL_COMPARE_NAME, compare_name,
-				-1);
-
-	    g_free (display_name);
-	    g_free (compare_name);
-	}
-	break;
-
-    case GWEATHER_LOCATION_WEATHER_STATION:
-	/* <location> with no parent <city>, or <city> with a single
-	 * child <location>.
-	 */
+	/* Merge the city name with the location-specific information. */
 	display_name = g_strdup_printf ("%s, %s",
 					gweather_location_get_name (loc),
 					parent_display_name);
@@ -373,7 +399,7 @@ fill_location_entry_model (GtkTreeStore *store, GWeatherLocation *loc,
 
 	gtk_tree_store_append (store, &iter, NULL);
 	gtk_tree_store_set (store, &iter,
-			    GWEATHER_LOCATION_ENTRY_COL_LOCATION, loc,
+			    GWEATHER_LOCATION_ENTRY_COL_LOCATION, children[0],
 			    GWEATHER_LOCATION_ENTRY_COL_DISPLAY_NAME, display_name,
 			    GWEATHER_LOCATION_ENTRY_COL_COMPARE_NAME, compare_name,
 			    -1);
@@ -381,6 +407,10 @@ fill_location_entry_model (GtkTreeStore *store, GWeatherLocation *loc,
 	g_free (display_name);
 	g_free (compare_name);
 	break;
+
+    case GWEATHER_LOCATION_WEATHER_STATION:
+	/* Shouldn't happen. Ignore */
+	break;
     }
 
     gweather_location_free_children (loc, children);
@@ -496,6 +526,18 @@ match_selected (GtkEntryCompletion *completion,
     return TRUE;
 }
 
+/**
+ * gweather_location_entry_new:
+ * @top: the top-level location for the entry.
+ *
+ * Creates a new #GWeatherLocationEntry.
+ *
+ * @top will normally be a location returned from
+ * gweather_location_new_world(), but you can create an entry that
+ * only accepts a smaller set of locations if you want.
+ *
+ * Return value: the new #GWeatherLocationEntry
+ **/
 GtkWidget *
 gweather_location_entry_new (GWeatherLocation *top)
 {
